@@ -1,19 +1,19 @@
+# Copyright (c) ONNX Project Contributors
+#
 # SPDX-License-Identifier: Apache-2.0
-
-from typing import Optional
+from __future__ import annotations
 
 import numpy as np
 
 import onnx
-
-from ..base import Base
-from . import expect
+from onnx.backend.test.case.base import Base
+from onnx.backend.test.case.node import expect
 
 
 def gemm_reference_implementation(
     A: np.ndarray,
     B: np.ndarray,
-    C: Optional[np.ndarray] = None,
+    C: np.ndarray | None = None,
     alpha: float = 1.0,
     beta: float = 1.0,
     transA: int = 0,
@@ -25,7 +25,7 @@ def gemm_reference_implementation(
 
     Y = alpha * np.dot(A, B) + beta * C
 
-    return Y
+    return Y.astype(A.dtype)
 
 
 class Gemm(Base):
