@@ -2,12 +2,8 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-#include "onnx/defs/function.h"
 #include "onnx/defs/schema.h"
 
-#include <algorithm>
-#include <numeric>
-
 namespace ONNX_NAMESPACE {
 static std::vector<std::string> optional_and_tensor_types() {
   auto optional_types = OpSchema::all_optional_types();
@@ -18,7 +14,7 @@ static std::vector<std::string> optional_and_tensor_types() {
   return optional_types;
 }
 
-static const char* Optional_ver15_doc = R"DOC(
+static constexpr const char* Optional_ver15_doc = R"DOC(
 Constructs an optional-type value containing either an empty optional of a certain type specified by the attribute,
 or a non-empty value containing the input element.
 )DOC";
@@ -70,7 +66,7 @@ ONNX_OPERATOR_SET_SCHEMA(
           }
         }));
 
-static const char* OptionalHasElement_ver18_doc = R"DOC(
+static constexpr const char* OptionalHasElement_ver18_doc = R"DOC(
 Returns true if (1) the input is an optional-type and contains an element,
 or, (2) the input is a tensor or sequence type.
 If the input is not provided or is an empty optional-type, this op returns false.
@@ -106,7 +102,7 @@ ONNX_OPERATOR_SET_SCHEMA(
           output_tensor_type->mutable_shape()->Clear();
         }));
 
-static const char* OptionalGetElement_ver18_doc = R"DOC(
+static constexpr const char* OptionalGetElement_ver18_doc = R"DOC(
 If the input is a tensor or sequence type, it returns the input.
 If the input is an optional type, it outputs the element in the input.
 It is an error if the input is an empty optional-type (i.e. does not have an element) and the behavior is undefined in this case.
