@@ -11,6 +11,8 @@
 #include <utility>
 #include <vector>
 
+#include "onnx/defs/tensor_proto_util.h"
+
 namespace ONNX_NAMESPACE {
 void resizeShapeInferenceHelper(
     const TensorShapeProto& input_shape,
@@ -135,7 +137,7 @@ void resizeShapeInferenceHelper(
     if (input_shape.dim(i).has_dim_value()) {
       int64_t dim_value =
           static_cast<int64_t>(std::floor(static_cast<float>(input_shape.dim(i).dim_value()) * scales_data[i]));
-      // If output_shape has dim_value, we validate the caculated result
+      // If output_shape has dim_value, we validate the calculated result
       // If output_shape doesn's have one, we set it to the scaled result
       if (dim->has_dim_value()) {
         if (static_cast<int64_t>(dim->dim_value()) != dim_value) {
@@ -147,13 +149,13 @@ void resizeShapeInferenceHelper(
               ").");
         }
       } else {
-        dim->set_dim_value(static_cast<int64_t>(dim_value));
+        dim->set_dim_value(dim_value);
       } // dim->has_dim_value()
     } // input_shape.dim(i).has_dim_value()
   }
 }
 
-void resizeShapeInferenceVersioned(InferenceContext& ctx, int opset_version) {
+static void resizeShapeInferenceVersioned(InferenceContext& ctx, int opset_version) {
   propagateElemTypeFromInputToOutput(ctx, 0, 0);
   if (!hasNInputShapes(ctx, 1)) {
     return;
@@ -334,7 +336,7 @@ void resizeShapeInferenceHelper_opset7_to_10(
     if (input_shape.dim(i).has_dim_value()) {
       int64_t dim_value =
           static_cast<int64_t>(std::floor(static_cast<float>(input_shape.dim(i).dim_value()) * scales_data[i]));
-      // If output_shape has dim_value, we validate the caculated result
+      // If output_shape has dim_value, we validate the calculated result
       // If output_shape doesn's have one, we set it to the scaled result
       if (dim->has_dim_value()) {
         if (static_cast<int64_t>(dim->dim_value()) != dim_value) {
@@ -346,7 +348,7 @@ void resizeShapeInferenceHelper_opset7_to_10(
               ").");
         }
       } else {
-        dim->set_dim_value(static_cast<int64_t>(dim_value));
+        dim->set_dim_value(dim_value);
       } // dim->has_dim_value()
     } // input_shape.dim(i).has_dim_value()
   }
@@ -379,7 +381,7 @@ void resizeShapeInference_opset7_to_10(InferenceContext& ctx) {
   if (nullptr != scales) {
     // Infer output shape's dimension value if 'scales' is known.
     if (scales->data_type() == TensorProto::FLOAT) {
-      const auto& scales_data = ParseData<float>(scales);
+      const auto scales_data = ParseData<float>(scales);
       if (scales_data.size() != static_cast<size_t>(input_shape.dim_size())) {
         fail_shape_inference("Number of elements of input 'scales' must be same as rank of input 'X'");
       }
@@ -393,8 +395,8 @@ void resizeShapeInference_opset7_to_10(InferenceContext& ctx) {
 std::function<void(OpSchema&)> PadDocGenerator(
     const char* description,
     const char* mode_description,
-    const std::vector<std::string> op_schema,
-    const std::string op_schema_description) {
+    const std::vector<std::string>& op_schema,
+    const std::string& op_schema_description) {
   return [=](OpSchema& schema) {
     schema.SetDoc(description);
     schema.Attr("mode", mode_description, AttributeProto::STRING, std::string("constant"));
@@ -464,7 +466,7 @@ std::function<void(OpSchema&)> PadDocGenerator(
         std::iota(axes.begin(), axes.end(), 0);
       }
 
-      int num_axes = axes.size();
+      auto num_axes = axes.size();
       auto* output_shape = ctx.getOutputType(0)->mutable_tensor_type()->mutable_shape();
 
       // Populating default dims
@@ -482,8 +484,8 @@ std::function<void(OpSchema&)> PadDocGenerator(
           fail_shape_inference("'pads' input must be a 1D (shape: [2 * num_axes]) tensor of type int64");
         }
 
-        const auto& pads_data = ParseData<int64_t>(pads_initializer);
-        if (pads_data.size() != static_cast<size_t>(2 * num_axes)) {
+        const auto pads_data = ParseData<int64_t>(pads_initializer);
+        if (pads_data.size() != 2 * num_axes) {
           fail_shape_inference(
               "Pads has incorrect number of values. Expected 2 * ",
               num_axes,
@@ -500,7 +502,7 @@ std::function<void(OpSchema&)> PadDocGenerator(
           }
         }
 
-        for (int i = 0; i < num_axes; ++i) {
+        for (size_t i = 0; i < num_axes; ++i) {
           auto axis = axes[i];
           const auto& input_dim = input_shape.dim(axis);
           auto& out_dim = *out_dims[axis];
