@@ -5,6 +5,7 @@
 
 This implements the python client for the ONNX model hub.
 """
+
 from __future__ import annotations
 
 import hashlib
@@ -13,7 +14,7 @@ import os
 import sys
 from io import BytesIO
 from os.path import join
-from typing import IO, Any, Dict, List, cast
+from typing import IO, Any, cast
 from urllib.error import HTTPError
 from urllib.request import urlopen
 
@@ -46,19 +47,21 @@ class ModelInfo:
         Args:
             raw_model_info: A JSON dict containing the model info.
         """
-        self.model = cast(str, raw_model_info["model"])
+        self.model = cast("str", raw_model_info["model"])
 
-        self.model_path = cast(str, raw_model_info["model_path"])
-        self.metadata: dict[str, Any] = cast(Dict[str, Any], raw_model_info["metadata"])
+        self.model_path = cast("str", raw_model_info["model_path"])
+        self.metadata: dict[str, Any] = cast(
+            "dict[str, Any]", raw_model_info["metadata"]
+        )
         self.model_sha: str | None = None
         if "model_sha" in self.metadata:
-            self.model_sha = cast(str, self.metadata["model_sha"])
+            self.model_sha = cast("str", self.metadata["model_sha"])
 
         self.tags: set[str] = set()
         if "tags" in self.metadata:
-            self.tags = set(cast(List[str], self.metadata["tags"]))
+            self.tags = set(cast("list[str]", self.metadata["tags"]))
 
-        self.opset = cast(int, raw_model_info["opset_version"])
+        self.opset = cast("int", raw_model_info["opset_version"])
         self.raw_model_info: dict[str, Any] = raw_model_info
 
     def __str__(self) -> str:
@@ -166,7 +169,7 @@ def list_models(
     try:
         with urlopen(manifest_url) as response:
             manifest: list[ModelInfo] = [
-                ModelInfo(info) for info in json.load(cast(IO[str], response))
+                ModelInfo(info) for info in json.load(cast("IO[str]", response))
             ]
     except HTTPError as e:
         raise AssertionError(f"Could not find manifest at {manifest_url}") from e
@@ -286,7 +289,7 @@ def load(
                 "download the model from the model hub."
             )
 
-    return onnx.load(cast(IO[bytes], BytesIO(model_bytes)))
+    return onnx.load(cast("IO[bytes]", BytesIO(model_bytes)))
 
 
 def download_model_with_test_data(
@@ -363,22 +366,18 @@ def download_model_with_test_data(
                 "download the model from the model hub."
             )
 
-    # FIXME: Avoid index manipulation with magic numbers,
-    # remove ".tar.gz"
     local_model_with_data_dir_path = local_model_with_data_path[
-        0 : len(local_model_with_data_path) - 7
+        0 : len(local_model_with_data_path) - len(".tar.gz")
     ]
     onnx.utils._extract_model_safe(
         local_model_with_data_path, local_model_with_data_dir_path
     )
-    model_with_data_path = (
+    return (
         local_model_with_data_dir_path
         + "/"
         + os.listdir(local_model_with_data_dir_path)[0]
     )
 
-    return model_with_data_path
-
 
 def load_composite_model(
     network_model: str,
@@ -473,10 +472,9 @@ def load_composite_model(
 
     io_map = [
         (out_entry.name, in_entry.name)
-        for out_entry, in_entry in zip(preprocessing.graph.output, network.graph.input)
+        for out_entry, in_entry in zip(
+            preprocessing.graph.output, network.graph.input, strict=False
+        )
     ]
 
-    model_with_preprocessing = onnx.compose.merge_models(
-        preprocessing, network, io_map=io_map
-    )
-    return model_with_preprocessing
+    return onnx.compose.merge_models(preprocessing, network, io_map=io_map)
