@@ -8,7 +8,7 @@ from onnx.reference.ops._op_common_random import _CommonRandom
 
 
 class Bernoulli(_CommonRandom):
-    def _run(self, x, dtype=None, seed=None):  # type: ignore
+    def _run(self, x, dtype=None, seed=None):
         if dtype is None:
             dtype = np_dtype_to_tensor_dtype(x.dtype)
         dtype = self._dtype(x, dtype=dtype, dtype_first=True)
