@@ -17,12 +17,11 @@ def reshape_reference_implementation(
     if allowzero == 0:
         zeros_index = np.where(shape == 0)
         new_shape[zeros_index] = np.array(data.shape)[zeros_index]
-    reshaped = np.reshape(data, new_shape)
-    return reshaped
+    return np.reshape(data, new_shape)
 
 
 class CommonReshape(OpRun):
-    def _run(self, data, shape):  # type: ignore
+    def _run(self, data, shape):
         return (reshape_reference_implementation(data, shape, 0),)
 
 
@@ -31,7 +30,7 @@ class Reshape_5(CommonReshape):
 
 
 class Reshape_14(CommonReshape):
-    def _run(self, data, shape, allowzero=None):  # type: ignore
+    def _run(self, data, shape, allowzero=None):
         if allowzero is None:
             allowzero = getattr(self, "allowzero", 0) == 1
         else:
