@@ -9,9 +9,11 @@ from onnx.reference.ops._op import OpRunUnaryNum
 
 
 class Softmax(OpRunUnaryNum):
-    def _run(self, X, axis=None):  # type: ignore
-        axis = axis or self.axis  # type: ignore
-        tmp = X - X.max(axis=axis, keepdims=1)  # type: ignore
+    def _run(self, X, axis=None):
+        if X.size == 0:
+            return (X,)
+        axis = axis or self.axis
+        tmp = X - X.max(axis=axis, keepdims=1)
         Y = np.exp(tmp)
-        Y /= Y.sum(axis=axis, keepdims=1)  # type: ignore
+        Y /= Y.sum(axis=axis, keepdims=1)
         return (Y.astype(X.dtype),)
