@@ -4,7 +4,7 @@
 from __future__ import annotations
 
 import unittest
-from typing import TYPE_CHECKING, Callable
+from typing import TYPE_CHECKING
 
 import numpy as np
 
@@ -24,7 +24,7 @@ from onnx import (
 )
 
 if TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import Callable, Sequence
 
 
 def _load_model(m_def: str) -> ModelProto:
@@ -393,7 +393,96 @@ class TestComposeFunctions(unittest.TestCase):
         m3 = compose.merge_models(m1, m2, io_map=io_map)
         checker.check_model(m3)
 
-    # FIXME: This function should be removed, as tests should not contain a copy of the tested logic.
+    def test_add_prefix_to_inputs_outputs(self) -> None:
+        """Tests prefixing inputs and outputs nodes."""
+        input_graph = """
+            <
+                ir_version: 6,
+                opset_import: [ "": 13]
+            >
+            agraph (float[N, 128] X, float[128, 10] W, float[10] B) => (float[N, 10] C)
+            {
+                T = MatMul(X, W)
+                S = Add(T, B)
+                C = Softmax(S)
+            }
+            """
+        input_model = _load_model(input_graph)
+        prefix = "pre_"
+        prefixed_model = compose.add_prefix(
+            model=input_model,
+            prefix=prefix,
+            rename_inputs=True,
+            rename_outputs=True,
+            rename_edges=True,
+        )
+
+        checker.check_graph(prefixed_model.graph)
+
+        for i in prefixed_model.graph.input:
+            self.assertTrue(i.name.startswith(prefix))
+        for o in prefixed_model.graph.output:
+            self.assertTrue(o.name.startswith(prefix))
+
+    def test_add_prefix_wo_inputs_outputs(self) -> None:
+        """Tests prefixing input and output nodes, when renaming of inputs/outputs is deactivated."""
+        input_graph = """
+            <
+                ir_version: 6,
+                opset_import: [ "": 13]
+            >
+            agraph (float[2,2] A) => (float[2,2] B)
+            {
+            B = Identity(A)
+            }
+            """
+        input_model = _load_model(input_graph)
+        prefix = "pre_"
+        prefixed_model = compose.add_prefix(
+            model=input_model,
+            prefix=prefix,
+            rename_inputs=False,
+            rename_outputs=False,
+            rename_edges=True,
+        )
+
+        checker.check_graph(prefixed_model.graph)
+        self.assertEqual(input_model.graph.input, prefixed_model.graph.input)
+        self.assertEqual(input_model.graph.output, prefixed_model.graph.output)
+
+    def test_add_prefix_with_loose_inputs_outputs(self) -> None:
+        """Tests prefixing of graphs with loose inputs."""
+        input_graph = """
+            <
+                ir_version: 7,
+                opset_import: [ "" : 13 ]
+            >
+            agraph (bool b, float[128] X, float[128] Y, bool unused_input) => (float[128] Z)
+            {
+            Z = If (b) <
+                then_branch = g1 () => (float[128] z_then) { z_then = Add(X, X) },
+                else_branch = g2 () => (float[128] z_else) { z_else = Sub(Y, Y) }
+                >
+            }
+            """
+        input_model = _load_model(input_graph)
+        prefix = "pre_"
+        prefixed_model = compose.add_prefix(
+            model=input_model,
+            prefix=prefix,
+            rename_inputs=True,
+            rename_outputs=True,
+            rename_edges=True,
+        )
+        checker.check_graph(prefixed_model.graph)
+
+        # Check if all inputs are prefixed even if input is not connected to node. e.g., 'unused_input'.
+        for i in prefixed_model.graph.input:
+            self.assertTrue(i.name.startswith(prefix))
+
+        for graph_output in prefixed_model.graph.output:
+            self.assertTrue(graph_output.name.startswith(prefix))
+
     def _test_add_prefix(
         self,
         rename_nodes: bool = False,
@@ -474,21 +563,21 @@ class TestComposeFunctions(unittest.TestCase):
                 for value_info in g_in.output:
                     name_mapping[value_info.name] = _prefixed(prefix, value_info.name)
 
-            for n1, n0 in zip(g_out.node, g_in.node):
-                for e1, e0 in zip(n1.input, n0.input):
+            for n1, n0 in zip(g_out.node, g_in.node, strict=True):
+                for e1, e0 in zip(n1.input, n0.input, strict=True):
                     self.assertEqual(name_mapping.get(e0, e0), e1)
-                for e1, e0 in zip(n1.output, n0.output):
+                for e1, e0 in zip(n1.output, n0.output, strict=True):
                     self.assertEqual(name_mapping.get(e0, e0), e1)
-            for i1, i0 in zip(g_out.input, g_in.input):
+            for i1, i0 in zip(g_out.input, g_in.input, strict=True):
                 self.assertEqual(name_mapping.get(i0.name, i0.name), i1.name)
-            for o1, o0 in zip(g_out.output, g_in.output):
+            for o1, o0 in zip(g_out.output, g_in.output, strict=True):
                 self.assertEqual(name_mapping.get(o0.name, o0.name), o1.name)
 
-            for init1, init0 in zip(g_out.initializer, g_in.initializer):
+            for init1, init0 in zip(g_out.initializer, g_in.initializer, strict=True):
                 self.assertEqual(name_mapping.get(init0.name, init0.name), init1.name)
 
             for sparse_init1, sparse_init0 in zip(
-                g_out.sparse_initializer, g_in.sparse_initializer
+                g_out.sparse_initializer, g_in.sparse_initializer, strict=True
             ):
                 self.assertEqual(
                     name_mapping.get(
@@ -503,11 +592,11 @@ class TestComposeFunctions(unittest.TestCase):
                     sparse_init1.indices.name,
                 )
 
-            for vi1, vi0 in zip(g_out.value_info, g_in.value_info):
+            for vi1, vi0 in zip(g_out.value_info, g_in.value_info, strict=True):
                 self.assertEqual(name_mapping.get(vi0.name, vi0.name), vi1.name)
 
             if rename_nodes:
-                for n1, n0 in zip(g_out.node, g_in.node):
+                for n1, n0 in zip(g_out.node, g_in.node, strict=True):
                     self.assertEqual(_prefixed(prefix, n0.name), n1.name)
 
     def test_add_prefix_nodes(self) -> None:
@@ -516,7 +605,9 @@ class TestComposeFunctions(unittest.TestCase):
 
     def test_add_prefix_edges(self) -> None:
         """Tests prefixing nodes edges. This will also rename inputs/outputs, since the names are shared"""
-        self._test_add_prefix(rename_edges=True)
+        self._test_add_prefix(
+            rename_edges=True, rename_inputs=True, rename_outputs=True
+        )
 
     def test_add_prefix_inputs(self) -> None:
         """Tests prefixing graph inputs only. Relevant node edges should be renamed as well"""
@@ -555,19 +646,19 @@ class TestComposeFunctions(unittest.TestCase):
         prefix = "prefix."
         prefixed_graph = compose.add_prefix_graph(graph, prefix)
         checker.check_graph(prefixed_graph)
-        for n1, n0 in zip(prefixed_graph.node, graph.node):
+        for n1, n0 in zip(prefixed_graph.node, graph.node, strict=True):
             self.assertEqual(_prefixed(prefix, n0.name), n1.name)
-            for attribute1, attribute0 in zip(n1.attribute, n0.attribute):
+            for attribute1, attribute0 in zip(n1.attribute, n0.attribute, strict=True):
                 if attribute1.g:
                     for subgraph_n1, subgraph_n0 in zip(
-                        attribute1.g.node, attribute0.g.node
+                        attribute1.g.node, attribute0.g.node, strict=True
                     ):
                         for input_n1, input_n0 in zip(
-                            subgraph_n1.input, subgraph_n0.input
+                            subgraph_n1.input, subgraph_n0.input, strict=True
                         ):
                             self.assertEqual(_prefixed(prefix, input_n0), input_n1)
                         for output_n1, output_n0 in zip(
-                            subgraph_n1.output, subgraph_n0.output
+                            subgraph_n1.output, subgraph_n0.output, strict=True
                         ):
                             self.assertEqual(_prefixed(prefix, output_n0), output_n1)
 
@@ -586,7 +677,7 @@ class TestComposeFunctions(unittest.TestCase):
         m1 = _load_model(M1_DEF)
 
         def _check_model(m1: ModelProto, m2: ModelProto, dim_idx: int) -> None:
-            for out_g2, out_g1 in zip(m2.graph.output, m1.graph.output):
+            for out_g2, out_g1 in zip(m2.graph.output, m1.graph.output, strict=True):
                 self.assertEqual(out_g2.name, out_g1.name)
                 self.assertEqual(
                     out_g2.type.tensor_type.elem_type, out_g1.type.tensor_type.elem_type
