@@ -7,13 +7,13 @@
 
 # export ONNX_ML=0, # if you need to disable ONNX_ML
 
-python_exist=`command -v python`
-if [ -z $python_exist ]; then
+python_exist=$(command -v python)
+if [[ -z $python_exist ]]; then
   echo "No python is found, please set it in your environment."
   exit 1
 fi
 
-if [ ! -f "onnx/defs/gen_doc.py" ]; then
+if [[ ! -f "onnx/defs/gen_doc.py" ]]; then
   echo "Please run this script in the ONNX root folder."
   exit 1
 fi
